//!
//!

use std::str::FromStr;

use super::super::super::compositions;
use super::super::super::repositories;
use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use diesel::{
  r2d2::{ConnectionManager, Pool},
  PgConnection,
};
use prost::Message;

///
pub struct Service<'a> {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
  pool: Pool<ConnectionManager<PgConnection>>,
  site_composition: compositions::site::composition::Composition,
  member_composition: compositions::member::composition::Composition,
  member_repository: repositories::member::repository::Repository,
  member_session_repository: repositories::member_session::repository::Repository,
  argon2_config: argon2::Config<'a>,
  password_salt: String,
}

impl std::fmt::Debug for Service<'_> {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Service of service.member.service.identity")
      .finish()
  }
}

impl Service<'_> {
  ///
  pub fn new(
    connection_broker: nats::asynk::Connection,
    queue_broker: String,
    pool: Pool<ConnectionManager<PgConnection>>,
    password_salt: String,
  ) -> Service<'static> {
    Service {
      connection_broker,
      queue_broker,
      pool,
      site_composition: compositions::site::composition::Composition::new(),
      member_composition: compositions::member::composition::Composition::new(),
      member_repository: repositories::member::repository::Repository::new(),
      member_session_repository: repositories::member_session::repository::Repository::new(),
      argon2_config: argon2::Config::default(),
      password_salt,
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(
      self.create_member(),
      self.list_members(),
      self.get_member(),
      self.get_member_by_username(),
      self.update_member(),
      self.delete_member(),
      self.event_after_signin()
    )
    .map(|_| ())
  }

  fn check_site(
    &self,
    conn: &diesel::PgConnection,
    url: Option<String>,
    site_id: uuid::Uuid,
  ) -> Result<repositories::site::models::Site, bcr::error::rpc::Error> {
    match self
      .site_composition
      .select_by_url(conn, url, site_id)
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })? {
      Some(s) => Ok(s),
      None => Err(bcr::error::rpc::Error::InvalidParams(
        bcr::error::rpc::InvalidParams {
          message: "invalid site_url information".to_string(),
          detail: bcr::error::rpc::InvalidParamsDetail {
            location: "request".to_string(),
            param: "client.site_url".to_string(),
            value: "".to_string(),
            error_type: bcr::error::rpc::InvalidParamsType::None,
            message: "".to_string(),
          },
        },
      )),
    }
  }

  async fn create_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::member::SUBJECT_CREATE_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::member::CreateMemberRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let site_id = uuid::Uuid::from_str(req.site_id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid site_id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "site_id".to_string(),
              value: req.site_id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;
        let member_class_id = uuid::Uuid::from_str(req.member_class_id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid member_class_id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "member_class_id".to_string(),
              value: req.member_class_id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;
        let member_level_id = uuid::Uuid::from_str(req.member_level_id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid member_level_id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "member_level_id".to_string(),
              value: req.member_level_id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let referrer_member_id = match req.referrer_member_username {
          Some(referrer_member_username) => {
            match self
              .member_repository
              .select_by_username(&conn, referrer_member_username.as_str())
              .map_err(|e| {
                bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                  code: bpr::protobuf::rpc::Error::SERVER_00,
                  message: format!("server {}", e),
                  data: None,
                })
              })? {
              Some(m) => Some(m.id),
              None => {
                return Err(bcr::error::rpc::Error::InvalidParams(
                  bcr::error::rpc::InvalidParams {
                    message: "invalid referrer_member_username information".to_string(),
                    detail: bcr::error::rpc::InvalidParamsDetail {
                      location: "request".to_string(),
                      param: "referrer_member_username".to_string(),
                      value: referrer_member_username.to_string(),
                      error_type: bcr::error::rpc::InvalidParamsType::None,
                      message: "not exist".to_string(),
                    },
                  },
                ));
              }
            }
          }
          None => None,
        };

        match self
          .member_repository
          .select_by_username(&conn, &req.username)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })? {
          Some(m) => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid username".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "username".to_string(),
                  value: req.username.to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::None,
                  message: "duplicated".to_string(),
                },
              },
            ));
          }
          None => {}
        };

        match self
          .member_repository
          .select_by_nickname(&conn, &req.nickname)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })? {
          Some(m) => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid nickname".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "nickname".to_string(),
                  value: req.nickname.to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::None,
                  message: "duplicated".to_string(),
                },
              },
            ));
          }
          None => {}
        };

        let password_hash = argon2::hash_encoded(
          req.password.as_bytes(),
          self.password_salt.as_bytes(),
          &self.argon2_config,
        )
        .map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let new_member = repositories::member::models::NewMember {
          site_id,
          member_class_id,
          member_level_id,
          referrer_member_id,
          username: req.username,
          password: password_hash,
          nickname: req.nickname,
          mobile_phone_number: req.mobile_phone_number,
        };

        let m = self
          .member_repository
          .insert(&conn, &new_member)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let member = self.member_composition.select(&conn, m.id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::member::CreateMemberResponse {
              error: None,
              result: Some(bpr::ss::member::member::create_member_response::Result {
                member: member.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::member::CreateMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn list_members(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::member::SUBJECT_LIST_MEMBERS,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::member::ListMembersRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let site_id = match req.site_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid site_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "site_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };
        let member_class_id = match req.member_class_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid member_class_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "member_class_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };
        let member_level_id = match req.member_level_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid member_level_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "member_level_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };
        let referrer_member_id = match req.referrer_member_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid referrer_member_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "referrer_member_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let find_all = repositories::member::models::FindAll {
          site_id,
          member_class_id,
          member_level_id,
          referrer_member_id,
          username_like: req.username_like,
          nickname_like: req.nickname_like,
          mobile_phone_number_like: req.mobile_phone_number_like,
          last_signined_ip: req.last_signined_ip,
          state: req
            .state
            .map(repositories::member::schema::MemberState::from),
          deleted_at: req.deleted_at.map(|d| d > 0),
          pagination: req
            .pagination
            .as_ref()
            .map(bcr::models::pagination::Pagination::from),
          sorts: Some(
            req
              .sorts
              .iter()
              .map(beteran_common_rust::models::pagination::Sort::from)
              .collect(),
          ),
        };

        let m = self
          .member_repository
          .select_all_count(&conn, &find_all)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let member_list = self
          .member_composition
          .select_all(&conn, &find_all)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::member::ListMembersResponse {
              error: None,
              result: Some(bpr::ss::member::member::list_members_response::Result {
                members: member_list
                  .iter()
                  .map(bpr::models::member::MemberModel::from)
                  .collect(),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::member::ListMembersResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::member::SUBJECT_GET_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::member::GetMemberRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let id = uuid::Uuid::from_str(req.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: req.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let m = self.member_composition.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::member::GetMemberResponse {
              error: None,
              result: Some(bpr::ss::member::member::get_member_response::Result {
                member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::member::GetMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member_by_username(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::member::SUBJECT_GET_MEMBER_BY_USERNAME,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req =
          bpr::ss::member::member::GetMemberByUsernameRequest::decode(message.data.as_slice())
            .map_err(|e| {
              bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
                message: format!("invalid request: {}", e),
              })
            })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let m = self
          .member_composition
          .select_by_username(&conn, req.username.as_str())
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::member::GetMemberByUsernameResponse {
              error: None,
              result: Some(
                bpr::ss::member::member::get_member_by_username_response::Result {
                  member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
                },
              ),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::member::GetMemberByUsernameResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::member::SUBJECT_UPDATE_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::member::UpdateMemberRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let id = uuid::Uuid::from_str(req.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: req.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let site_id = match req.site_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid site_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "site_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };

        let member_level_id = match req.member_level_id {
          Some(d) => match uuid::Uuid::from_str(d.as_str()) {
            Ok(dd) => Some(dd),
            Err(e) => {
              return Err(bcr::error::rpc::Error::InvalidParams(
                bcr::error::rpc::InvalidParams {
                  message: "invalid member_level_id param".to_string(),
                  detail: bcr::error::rpc::InvalidParamsDetail {
                    location: "request".to_string(),
                    param: "member_level_id".to_string(),
                    value: d.clone(),
                    error_type: bcr::error::rpc::InvalidParamsType::Required,
                    message: e.to_string(),
                  },
                },
              ));
            }
          },
          None => None,
        };

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let modify_member = repositories::member::models::ModifyMember {
          site_id,
          member_level_id,
          password: req.password,
          mobile_phone_number: req.mobile_phone_number,
          state: req
            .state
            .map(repositories::member::schema::MemberState::from),
        };

        let _affected = self
          .member_repository
          .update(&conn, id, &modify_member)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let m = self.member_composition.select(&conn, id).map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        message
          .respond(
            bpr::ss::member::member::UpdateMemberResponse {
              error: None,
              result: Some(bpr::ss::member::member::update_member_response::Result {
                member: m.map(|d| bpr::models::member::MemberModel::from(&d)),
              }),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::member::UpdateMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn delete_member(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::member::SUBJECT_DELETE_MEMBER,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let req = bpr::ss::member::member::DeleteMemberRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let id = uuid::Uuid::from_str(req.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "id".to_string(),
              value: req.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let client = match req.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let deleted_at = chrono::Utc::now().timestamp();

        let modify_member = repositories::member::models::ModifyMember4DeletedAt {
          deleted_at: Some(deleted_at),
        };

        let m = self
          .member_repository
          .update_deleted_at(&conn, id, &modify_member)
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        message
          .respond(
            bpr::ss::member::member::DeleteMemberResponse {
              error: None,
              result: Some(bpr::ss::member::member::delete_member_response::Result {}),
            }
            .encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::ss::member::member::DeleteMemberResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn event_after_signin(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::ss::member::identity::EVENT_SUBJECT_AFTER_SIGNIN,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let event = bpr::ss::member::identity::AfterSigninEvent::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let client = match event.client {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let member = match event.member {
          Some(c) => c,
          None => {
            return Err(bcr::error::rpc::Error::InvalidParams(
              bcr::error::rpc::InvalidParams {
                message: "invalid client information".to_string(),
                detail: bcr::error::rpc::InvalidParamsDetail {
                  location: "request".to_string(),
                  param: "client".to_string(),
                  value: "".to_string(),
                  error_type: bcr::error::rpc::InvalidParamsType::Required,
                  message: "".to_string(),
                },
              },
            ));
          }
        };

        let id = uuid::Uuid::from_str(member.id.as_str()).map_err(|e| {
          bcr::error::rpc::Error::InvalidParams(bcr::error::rpc::InvalidParams {
            message: "invalid member.id param".to_string(),
            detail: bcr::error::rpc::InvalidParamsDetail {
              location: "request".to_string(),
              param: "member.id".to_string(),
              value: member.id.clone(),
              error_type: bcr::error::rpc::InvalidParamsType::Required,
              message: e.to_string(),
            },
          })
        })?;

        let conn = self.pool.get().map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        let last_signined_at = chrono::Utc::now().timestamp();

        self
          .member_repository
          .update_last_signined_ip(
            &conn,
            id,
            &repositories::member::models::ModifyMember4LastSignined {
              last_signined_ip: client.client_ip,
              last_signined_at,
            },
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        println!("error: {}", e);
      }
    }

    Ok(())
  }
}
